@@ -32,6 +32,7 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
   private String password;
   private boolean enableAudio;
   private boolean enableVideo;
+  private boolean keepAspect;
   private boolean ignoreAspect;
   private int enableKate;
   private Component component;
@@ -48,7 +49,7 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
   private Element audiodec;
   private Element videosink;
   private Element audiosink;
-  private Element v_queue, v_queue2, a_queue;
+  private Element v_queue, v_queue2, a_queue = null;
   private Element overlay;
   private Pad asinkpad, ovsinkpad, oksinkpad;
   private Pad apad, vpad;
@@ -80,18 +81,31 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
     String mime = caps.getMime();
     
     if (enableAudio && mime.equals("audio/x-vorbis")) {
+      if (a_queue != null) {
+        Debug.log (Debug.INFO, "More than one audio stream detected, ignoring all except first one");
+        return;
+      }
+
       a_queue = ElementFactory.makeByName("queue", "a_queue");
       if (a_queue == null) {
         noSuchElement ("queue");
         return;
       }
 
+      // if we already have a video queue: We want smooth audio playback
+      // over frame completeness, so make the video queue leaky
+      if(v_queue != null) {
+        v_queue.setProperty("leaky", "2"); // 2 == Queue.LEAK_DOWNSTREAM
+      }
+
       audiodec = ElementFactory.makeByName("vorbisdec", "audiodec");
       if (audiodec == null) {
         noSuchElement ("vorbisdec");
         return;
       }
 
+      a_queue.setProperty("maxBuffers","100");
+
       add(a_queue);
       add(audiodec);
 
@@ -120,7 +134,12 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
       if (!setupVideoDec ("theoradec"))
         return;
 
-      v_queue.setProperty("leaky", "2"); // 2 == Queue.LEAK_DOWNSTREAM
+      // if we have audio: We want smooth audio playback
+      // over frame completeness
+      if(a_queue != null) {
+        v_queue.setProperty("leaky", "2"); // 2 == Queue.LEAK_DOWNSTREAM
+      }
+      v_queue.setProperty("maxBuffers","175");
       v_queue2.setProperty("maxBuffers","1");
 
       add(v_queue);
@@ -338,6 +357,9 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
   public void setUserId(String aUserId) {
     userId = aUserId;
   }
+  public void setKeepAspect(boolean keep) {
+    keepAspect = keep;
+  }
   public void setIgnoreAspect(boolean ignore) {
     ignoreAspect = ignore;
   }
@@ -599,6 +621,7 @@ public class CortadoPipeline extends Pipeline implements PadListener, CapsListen
         return false;
       }
 
+      videosink.setProperty("keep-aspect", keepAspect ? "true" : "false");
       videosink.setProperty("ignore-aspect", ignoreAspect ? "true" : "false");
 
       videosink.setProperty ("component", component);
