@@ -191,7 +191,7 @@ public class TheoraDec extends Element implements OggPayload
 	return OK;
       }
       else {
-        if ((op.packet_base[op.packet] & 0x80) == 0x80) {
+        if (op.packet_base.length > 0 && (op.packet_base[op.packet] & 0x80) == 0x80) {
           Debug.log(Debug.INFO, "ignoring header");
           return OK;
         }
